/*  

    Panda -  A modular programming language.
    written by Giovanni Petrantoni.
    Copyright (c) 2003 - 2004, Giovanni Petrantoni.
    http://xpanda.sf.net/
    
    panda_modules.h - Panda modules API
    
    This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.


	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	Lesser General Public License for more details.


	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
	
	elementicaotici - by Johnny (Giovanni) Petrantoni, ITALY - Rome.
	e-mail: johnny@lato-b.com
	
*/


#include <stdio.h>
#ifdef __APPLE__
#include <CoreFoundation/CoreFoundation.h>
#elif __WIN32__
#include <windows.h>
#endif

enum { //Variable types, (varType);
    INT_T,CHAR_T,FLOAT_T,DOUBLE_T,BYTE_T,ULONG_T,LONG_T,LDOUBLE_T,OBJECT_T,PTR_T,BANG_T
};

enum {
    B_YES = 1, B_NO = 2
};

enum { //Module types, (TYPE);
    AUDIO_PARAMETERS = 1, ONLY_PARAMETERS = 3
};

typedef unsigned char P_BOOL;
typedef unsigned char P_BYTE;
typedef int P_ID; //module ID type (please use this type in your modules, because int will be changed to UInt32)

typedef struct _inletBuf {
    float *buffers[256];
    int manyBuffers;
} BuffersList;


typedef struct _variable {
    int varType;
    void *var_t;
    int manyVar;
    int length;
    char *specification;
} Var;

typedef int TYPE;

#ifndef __WIN32__
typedef void * (*ThreadFunctionPtr)(void *x);
#define PANDA_THREAD void * 
#define PANDA_THREAD_RETURN return NULL
#else
typedef unsigned long (WINAPI *ThreadFunctionPtr)(void*);
#define PANDA_THREAD unsigned long WINAPI
#define PANDA_THREAD_RETURN return 0
#endif

struct __myObjData; //this is your data structure (YOU MUST DEFINE IT)

typedef struct __myObjData MyData; //this is your data structure (YOU MUST DEFINE IT)

typedef struct _parametersList {
    TYPE moduleType;
    int subType; //use 'coco' for cocoa modules (that kind of bundle cannot be completely released, btw to be defined)
    int outlets;
    int inlets;
    int moduleID;
    int audio_outlets; //olny if AUDIO type is declared, 1 outlet is 1 audio output channel
    int audio_inlets; //olny if AUDIO type is declared, 1 inlet is 1 audio input channel
    P_BOOL audio_startpoint; //if your AUDIO module has no audio input inlets it will need an audio_startpoint
}ParamList;

#ifdef __cplusplus
extern "C" {
#endif

extern void outletCall(int ID, int outletNumber,void * values);
extern int audio_StartPointCall(int ID, int outletNumber,long bufferSize);
extern int audio_DoRenderAtModule(int ID,long bufferSize);
extern BuffersList GetOutletBuffers(int ID, int outletNumber);
extern float **GetADCBuffers(int *channels);
extern float **GetDACBuffers(int *channels);
extern long GetPandaBufferSize(void);
inline static long GetBufferSize(void) {
	return GetPandaBufferSize();
}
extern void SetPandaBufferSize(long newNFrames); //please use this only for multiple buffer size zones management.
extern float GetSampleRate(void);
extern int GetDeviceInputChannels(void);
extern int GetDeviceOutputChannels(void);
#ifdef __APPLE__
extern CFBundleRef GetModuleBundleFromID(int ID);
#else
extern void * GetModuleBundleFromID(int ID);
#endif
extern char * GetModuleResourcesPath(int ID);
extern void * GetModuleVoidPtr(int ID);
extern void PandaLog(char *log,...);
extern int setSharedPointer(char *key,void *ptr); 
extern void *sharedPointer(char *key); 
extern void removeSharedPointer(char *key);
extern float **GetPluginADCBuffers(char *instanceName);
extern float **GetPluginDACBuffers(char *instanceName);
extern P_BOOL GetPluginAudioConfiguration(char *instanceName,float *outSR,long *outBSize,int *outInCh,int *outOutCh);

//UTILITIES
extern Var *initNewVar(int *types,int many,void **values);
extern int c_bool(P_BOOL the_p_bool);
extern P_BOOL p_bool(int c_cond);

//THREADS
int addThread(void *callerData,ThreadFunctionPtr ThreadFunction); //Round-robin thread with priority 62.
void removeThread(int ThreadReference); //You must exit from the thread function before to call this.
int changeThreadPriority(int ThreadReference,int priority);

#ifdef __cplusplus
}
#endif

//REQUIRED METHODS
#ifdef __cplusplus
extern "C" {
#endif

MyData * HelloPanda (int *err,int MODULE_ID,int argc,char **argv);
void ShowHelp(void);
void InletCallback (MyData *x,int inlet, Var *inValues);
float ** AllocInlets (MyData *x,long bufferSize, int *manyBuffers);
int AudioProcess (MyData *x,long nframes); //only for audio modules
ParamList GetMethodsList (MyData *x);
void FlushModule (MyData *x);

#ifdef __cplusplus
}
#endif

    
